/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* 
* Copyright (C) 2015-2019 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/

/******************************************************************************
* File Name   : mtr_ctrl_rl78g1f_t2001.c
* Description : The processing of a H/W control layer(MCU dependance)
******************************************************************************/
/******************************************************************************
* History : 23.03.2015  Ver.1.00
******************************************************************************/

/******************************************************************************
* Includes <System Includes> , "Project Includes"
******************************************************************************/
#include "mtr_common.h"
#include "r_dsp.h"
#include "mtr_ctrl_rl78g1f.h"
#include "mtr_ctrl_rl78g1f_t2001.h"
#include "mtr_spm_hall_120_cpm.h"

/******************************************************************************
* Exported global variables
******************************************************************************/
extern volatile int16_t    g_s2_pwm_duty;                               /* PWM duty (setting value of PWM timer compare register) */

/* for sequence control */
extern volatile uint8_t    g_u1_error_status;                           /* error status */

/******************************************************************************
* Function Name : mtr_ctrl_start
* Description   : hall sensor interrupt enable for motor control start
* Arguments     : none
* Return Value  : none
******************************************************************************/
void mtr_ctrl_start(void)
{
    PMK1 = 0;                                                           /* INTP1 interrupt enable (HU) */
    PMK2 = 0;                                                           /* INTP2 interrupt enable (HV) */
    PMK3 = 0;                                                           /* INTP3 interrupt enable (HW) */
}

/******************************************************************************
* Function Name : mtr_ctrl_stop
* Description   : TRD output disable, hall sensor interrupt disable,
*                 motor output port is on/off for motor control stop
* Arguments     : none
* Return Value  : none
******************************************************************************/
void mtr_ctrl_stop(void)
{
    TRDOER1 = 0xFF;                                                     /* TRD output disable */

    MTR_PORT_UP = 0;                                                    /* Up = OFF("Low") */
    MTR_PORT_UN = 1;                                                    /* Un = ON("High") */
    MTR_PORT_VP = 0;                                                    /* Vp = OFF("Low") */
    MTR_PORT_VN = 1;                                                    /* Vn = ON("High") */
    MTR_PORT_WP = 0;                                                    /* Wp = OFF("Low") */
    MTR_PORT_WN = 1;                                                    /* Wn = ON("High") */

    PMK1 = 1;                                                           /* INTP1 interrupt disable (HU) */
    PMK2 = 1;                                                           /* INTP2 interrupt disable (HV) */
    PMK3 = 1;                                                           /* INTP3 interrupt disable (HW) */
}

/******************************************************************************
* Function Name : mtr_change_pattern
* Description   : Change voltage pattern
* Arguments     : pattern - Voltage pattern
* Return Value  : none
******************************************************************************/
void mtr_change_pattern(uint8_t pattern)
{
    /*========================*/
    /*     stop PWM timer     */
    /*========================*/

    TRDSTR = 0x0C;                                                      /* stop TRD0 */

    /*======================*/
    /*     set register     */
    /*======================*/

    switch (pattern)
    {
        case MTR_VP_ON_W_PWM:
            TRDOER1 = 0x5F;                                             /* Wn : PWM output enable */
            MTR_PORT_UP = 0;                                            /* Up = OFF("Low") */
            MTR_PORT_UN = 0;                                            /* Un = OFF("Low") */
            MTR_PORT_VP = 1;                                            /* Vp = ON ("High") */
            MTR_PORT_VN = 0;                                            /* Vn = OFF("Low") */
            MTR_PORT_WP = 0;                                            /* Wp = OFF("Low") */
            MTR_PORT_WN = 0;                                            /* Wn = OFF("Low") */
            TRDGRD0 = g_s2_pwm_duty;                                    /* duty of U phase */
            TRDGRC1 = g_s2_pwm_duty;                                    /* duty of V phase */
            TRDGRD1 = g_s2_pwm_duty;                                    /* duty of W phase */
            TRDGRB0 = g_s2_pwm_duty;                                    /* duty of U phase */
            TRDGRA1 = g_s2_pwm_duty;                                    /* duty of V phase */
            TRDGRB1 = g_s2_pwm_duty;                                    /* duty of W phase */
        break;

        case MTR_W_PWM_VN_ON:
            TRDOER1 = 0x5F;                                             /* Wp : PWM output enable */
            MTR_PORT_UP = 0;                                            /* Up = OFF("Low") */
            MTR_PORT_UN = 0;                                            /* Un = OFF("Low") */
            MTR_PORT_VP = 0;                                            /* Vp = OFF("Low") */
            MTR_PORT_VN = 1;                                            /* Vn = ON ("High") */
            MTR_PORT_WP = 0;                                            /* Wp = OFF("Low") */
            MTR_PORT_WN = 0;                                            /* Wn = OFF("Low") */
            /* intentionally casting float to int16_t */
            TRDGRD0 = (int16_t)MTR_CARRIER_SET - g_s2_pwm_duty;         /* duty of U phase */
            TRDGRC1 = (int16_t)MTR_CARRIER_SET - g_s2_pwm_duty;         /* duty of V phase */
            TRDGRD1 = (int16_t)MTR_CARRIER_SET - g_s2_pwm_duty;         /* duty of W phase */
            TRDGRB0 = (int16_t)MTR_CARRIER_SET - g_s2_pwm_duty;         /* duty of U phase */
            TRDGRA1 = (int16_t)MTR_CARRIER_SET - g_s2_pwm_duty;         /* duty of V phase */
            TRDGRB1 = (int16_t)MTR_CARRIER_SET - g_s2_pwm_duty;         /* duty of W phase */
        break;

        case MTR_WP_ON_U_PWM:
            TRDOER1 = 0xF5;                                             /* Un : PWM output enable */
            MTR_PORT_UP = 0;                                            /* Up = OFF("Low") */
            MTR_PORT_UN = 0;                                            /* Un = OFF("Low") */
            MTR_PORT_VP = 0;                                            /* Vp = OFF("Low") */
            MTR_PORT_VN = 0;                                            /* Vn = OFF("Low") */
            MTR_PORT_WP = 1;                                            /* Wp = ON ("High") */
            MTR_PORT_WN = 0;                                            /* Wn = OFF("Low") */
            TRDGRD0 = g_s2_pwm_duty;                                    /* duty of U phase */
            TRDGRC1 = g_s2_pwm_duty;                                    /* duty of V phase */
            TRDGRD1 = g_s2_pwm_duty;                                    /* duty of W phase */
            TRDGRB0 = g_s2_pwm_duty;                                    /* duty of U phase */
            TRDGRA1 = g_s2_pwm_duty;                                    /* duty of V phase */
            TRDGRB1 = g_s2_pwm_duty;                                    /* duty of W phase */
        break;

        case MTR_U_PWM_WN_ON:
            TRDOER1 = 0xF5;                                             /* Up : PWM output enable */
            MTR_PORT_UP = 0;                                            /* Up = OFF("Low") */
            MTR_PORT_UN = 0;                                            /* Un = OFF("Low") */
            MTR_PORT_VP = 0;                                            /* Vp = OFF("Low") */
            MTR_PORT_VN = 0;                                            /* Vn = OFF("Low") */
            MTR_PORT_WP = 0;                                            /* Wp = OFF("Low") */
            MTR_PORT_WN = 1;                                            /* Wn = ON ("High") */
            /* intentionally casting float to int16_t */
            TRDGRD0 = (int16_t)MTR_CARRIER_SET - g_s2_pwm_duty;         /* duty of U phase */
            TRDGRC1 = (int16_t)MTR_CARRIER_SET - g_s2_pwm_duty;         /* duty of V phase */
            TRDGRD1 = (int16_t)MTR_CARRIER_SET - g_s2_pwm_duty;         /* duty of W phase */
            TRDGRB0 = (int16_t)MTR_CARRIER_SET - g_s2_pwm_duty;         /* duty of U phase */
            TRDGRA1 = (int16_t)MTR_CARRIER_SET - g_s2_pwm_duty;         /* duty of V phase */
            TRDGRB1 = (int16_t)MTR_CARRIER_SET - g_s2_pwm_duty;         /* duty of W phase */
        break;

        case MTR_UP_ON_V_PWM:
            TRDOER1 = 0xAF;                                             /* Vn : PWM output enable */
            MTR_PORT_UP = 1;                                            /* Up = ON ("High") */
            MTR_PORT_UN = 0;                                            /* Un = OFF("Low") */
            MTR_PORT_VP = 0;                                            /* Vp = OFF("Low") */
            MTR_PORT_VN = 0;                                            /* Vn = OFF("Low") */
            MTR_PORT_WP = 0;                                            /* Wp = OFF("Low") */
            MTR_PORT_WN = 0;                                            /* Wn = OFF("Low") */
            TRDGRD0 = g_s2_pwm_duty;                                    /* duty of U phase */
            TRDGRC1 = g_s2_pwm_duty;                                    /* duty of V phase */
            TRDGRD1 = g_s2_pwm_duty;                                    /* duty of W phase */
            TRDGRB0 = g_s2_pwm_duty;                                    /* duty of U phase */
            TRDGRA1 = g_s2_pwm_duty;                                    /* duty of V phase */
            TRDGRB1 = g_s2_pwm_duty;                                    /* duty of W phase */
        break;

        case MTR_V_PWM_UN_ON:
            TRDOER1 = 0xAF;                                             /* Vp : PWM output enable */
            MTR_PORT_UP = 0;                                            /* Up = OFF("Low") */
            MTR_PORT_UN = 1;                                            /* Un = ON ("High") */
            MTR_PORT_VP = 0;                                            /* Vp = OFF("Low") */
            MTR_PORT_VN = 0;                                            /* Vn = OFF("Low") */
            MTR_PORT_WP = 0;                                            /* Wp = OFF("Low") */
            MTR_PORT_WN = 0;                                            /* Wn = OFF("Low") */
            /* intentionally casting float to int16_t */
            TRDGRD0 = (int16_t)MTR_CARRIER_SET - g_s2_pwm_duty;         /* duty of U phase */
            TRDGRC1 = (int16_t)MTR_CARRIER_SET - g_s2_pwm_duty;         /* duty of V phase */
            TRDGRD1 = (int16_t)MTR_CARRIER_SET - g_s2_pwm_duty;         /* duty of W phase */
            TRDGRB0 = (int16_t)MTR_CARRIER_SET - g_s2_pwm_duty;         /* duty of U phase */
            TRDGRA1 = (int16_t)MTR_CARRIER_SET - g_s2_pwm_duty;         /* duty of V phase */
            TRDGRB1 = (int16_t)MTR_CARRIER_SET - g_s2_pwm_duty;         /* duty of W phase */
        break;

        case MTR_U_PWM_VN_ON:
            TRDOER1 = 0xF5;                                             /* Up : PWM output enable */
            MTR_PORT_UP = 0;                                            /* Up = OFF("Low") */
            MTR_PORT_UN = 0;                                            /* Un = OFF("Low") */
            MTR_PORT_VP = 0;                                            /* Vp = OFF("Low") */
            MTR_PORT_VN = 1;                                            /* Vn = ON ("High") */
            MTR_PORT_WP = 0;                                            /* Wp = OFF("Low") */
            MTR_PORT_WN = 0;                                            /* Wn = OFF("Low") */
            /* intentionally casting float to int16_t */
            TRDGRD0 = (int16_t)MTR_CARRIER_SET - g_s2_pwm_duty;         /* duty of U phase */
            TRDGRC1 = (int16_t)MTR_CARRIER_SET - g_s2_pwm_duty;         /* duty of V phase */
            TRDGRD1 = (int16_t)MTR_CARRIER_SET - g_s2_pwm_duty;         /* duty of W phase */
            TRDGRB0 = (int16_t)MTR_CARRIER_SET - g_s2_pwm_duty;         /* duty of U phase */
            TRDGRA1 = (int16_t)MTR_CARRIER_SET - g_s2_pwm_duty;         /* duty of V phase */
            TRDGRB1 = (int16_t)MTR_CARRIER_SET - g_s2_pwm_duty;         /* duty of W phase */
        break;

        case MTR_VP_ON_U_PWM:
            TRDOER1 = 0xF5;                                             /* Un : PWM output enable */
            MTR_PORT_UP = 0;                                            /* Up = OFF("Low") */
            MTR_PORT_UN = 0;                                            /* Un = OFF("Low") */
            MTR_PORT_VP = 1;                                            /* Vp = ON ("High") */
            MTR_PORT_VN = 0;                                            /* Vn = OFF("Low") */
            MTR_PORT_WP = 0;                                            /* Wp = OFF("Low") */
            MTR_PORT_WN = 0;                                            /* Wn = OFF("Low") */
            TRDGRD0 = g_s2_pwm_duty;                                    /* duty of U phase */
            TRDGRC1 = g_s2_pwm_duty;                                    /* duty of V phase */
            TRDGRD1 = g_s2_pwm_duty;                                    /* duty of W phase */
            TRDGRB0 = g_s2_pwm_duty;                                    /* duty of U phase */
            TRDGRA1 = g_s2_pwm_duty;                                    /* duty of V phase */
            TRDGRB1 = g_s2_pwm_duty;                                    /* duty of W phase */
        break;

        case MTR_V_PWM_WN_ON:
            TRDOER1 = 0xAF;                                             /* Vp : PWM output enable */
            MTR_PORT_UP = 0;                                            /* Up = OFF("Low") */
            MTR_PORT_UN = 0;                                            /* Un = OFF("Low") */
            MTR_PORT_VP = 0;                                            /* Vp = OFF("Low") */
            MTR_PORT_VN = 0;                                            /* Vn = OFF("Low") */
            MTR_PORT_WP = 0;                                            /* Wp = OFF("Low") */
            MTR_PORT_WN = 1;                                            /* Wn = ON ("High") */
            /* intentionally casting float to int16_t */
            TRDGRD0 = (int16_t)MTR_CARRIER_SET - g_s2_pwm_duty;         /* duty of U phase */
            TRDGRC1 = (int16_t)MTR_CARRIER_SET - g_s2_pwm_duty;         /* duty of V phase */
            TRDGRD1 = (int16_t)MTR_CARRIER_SET - g_s2_pwm_duty;         /* duty of W phase */
            TRDGRB0 = (int16_t)MTR_CARRIER_SET - g_s2_pwm_duty;         /* duty of U phase */
            TRDGRA1 = (int16_t)MTR_CARRIER_SET - g_s2_pwm_duty;         /* duty of V phase */
            TRDGRB1 = (int16_t)MTR_CARRIER_SET - g_s2_pwm_duty;         /* duty of W phase */
        break;

        case MTR_WP_ON_V_PWM:
            TRDOER1 = 0xAF;                                             /* Vn : PWM output enable */
            MTR_PORT_UP = 0;                                            /* Up = OFF("Low") */
            MTR_PORT_UN = 0;                                            /* Un = OFF("Low") */
            MTR_PORT_VP = 0;                                            /* Vp = OFF("Low") */
            MTR_PORT_VN = 0;                                            /* Vn = OFF("Low") */
            MTR_PORT_WP = 1;                                            /* Wp = ON ("High") */
            MTR_PORT_WN = 0;                                            /* Wn = OFF("Low") */
            TRDGRD0 = g_s2_pwm_duty;                                    /* duty of U phase */
            TRDGRC1 = g_s2_pwm_duty;                                    /* duty of V phase */
            TRDGRD1 = g_s2_pwm_duty;                                    /* duty of W phase */
            TRDGRB0 = g_s2_pwm_duty;                                    /* duty of U phase */
            TRDGRA1 = g_s2_pwm_duty;                                    /* duty of V phase */
            TRDGRB1 = g_s2_pwm_duty;                                    /* duty of W phase */
        break;

        case MTR_W_PWM_UN_ON:
            TRDOER1 = 0x5F;                                             /* Wp : PWM output enable */
            MTR_PORT_UP = 0;                                            /* Up = OFF("Low") */
            MTR_PORT_UN = 1;                                            /* Un = ON ("High") */
            MTR_PORT_VP = 0;                                            /* Vp = OFF("Low") */
            MTR_PORT_VN = 0;                                            /* Vn = OFF("Low") */
            MTR_PORT_WP = 0;                                            /* Wp = OFF("Low") */
            MTR_PORT_WN = 0;                                            /* Wn = OFF("Low") */
            /* intentionally casting float to int16_t */
            TRDGRD0 = (int16_t)MTR_CARRIER_SET - g_s2_pwm_duty;         /* duty of U phase */
            TRDGRC1 = (int16_t)MTR_CARRIER_SET - g_s2_pwm_duty;         /* duty of V phase */
            TRDGRD1 = (int16_t)MTR_CARRIER_SET - g_s2_pwm_duty;         /* duty of W phase */
            TRDGRB0 = (int16_t)MTR_CARRIER_SET - g_s2_pwm_duty;         /* duty of U phase */
            TRDGRA1 = (int16_t)MTR_CARRIER_SET - g_s2_pwm_duty;         /* duty of V phase */
            TRDGRB1 = (int16_t)MTR_CARRIER_SET - g_s2_pwm_duty;         /* duty of W phase */
        break;

        case MTR_UP_ON_W_PWM:
            TRDOER1 = 0x5F;                                             /* Wn : PWM output enable */
            MTR_PORT_UP = 1;                                            /* Up = ON ("High") */
            MTR_PORT_UN = 0;                                            /* Un = OFF("Low") */
            MTR_PORT_VP = 0;                                            /* Vp = OFF("Low") */
            MTR_PORT_VN = 0;                                            /* Vn = OFF("Low") */
            MTR_PORT_WP = 0;                                            /* Wp = OFF("Low") */
            MTR_PORT_WN = 0;                                            /* Wn = OFF("Low") */
            TRDGRD0 = g_s2_pwm_duty;                                    /* duty of U phase */
            TRDGRC1 = g_s2_pwm_duty;                                    /* duty of V phase */
            TRDGRD1 = g_s2_pwm_duty;                                    /* duty of W phase */
            TRDGRB0 = g_s2_pwm_duty;                                    /* duty of U phase */
            TRDGRA1 = g_s2_pwm_duty;                                    /* duty of V phase */
            TRDGRB1 = g_s2_pwm_duty;                                    /* duty of W phase */
        break;

        /* the default case is intentionally combined */
        case MTR_PATTERN_ERROR:
        default:
#ifdef FW_ERR_DET_HALL
            R_MTR_ExecEvent(MTR_EVENT_ERROR);                           /* execute reset event */

            DI();

            g_u1_error_status |= MTR_HALL_ERROR;                        /* error status update : hall error */

            EI();
#endif
        break;
    }

    /*===========================*/
    /*     restart PWM timer     */
    /*===========================*/

    TRD0 = (uint16_t)MTR_DEADTIME_SET;
    TRD1 = 0;

    TRDSTR = 0x0F;                                                      /* start TRD0 */
}

/******************************************************************************
* Function Name : mtr_get_adc
* Description   : A/D conversion
* Arguments     : ad_ch - A/D converter channel number
* Return Value  : A/D converted value
******************************************************************************/
int16_t mtr_get_adc(uint8_t ad_ch)
{
    int16_t s2_temp;

    ADS = ad_ch;                                                        /* set converted channel */

    ADCS = 1;

    while (ADIF == 0)
    {
        /* waiting for A/D converter */
    }

    s2_temp = ((ADCR >> 6) & 0x03FF);                                   /* get A/D converted value (10bit) */

    ADIF = 0;

    return (s2_temp);
}

