/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
* 
* Copyright (C) 2015-2019 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/

/******************************************************************************
* File Name   : mtr_spm_hall_120_cpm.h
* Description : A definitions of the processing of a motor control layer
******************************************************************************/
/******************************************************************************
* History : 23.03.2015  Ver.1.00
******************************************************************************/

/* guard against multiple inclusion */
#ifndef MTR_SPM_HALL_120_CPM_H
#define MTR_SPM_HALL_120_CPM_H

/******************************************************************************
* Macro definitions
******************************************************************************/
#define MTR_TWOPI               (2 * 3.14159265f)                       /* 2 * pi */
#define MTR_POLE_PAIRS          (MP_POLE_PAIRS)                         /* number of pole pairs */
#define MTR_RPM_RAD             (1716)                                  /* [rpm]->[rad/s] : scale(2^12 * 2^2 = 2^14) */
#define MTR_RAD_RPM             (2445 / MTR_POLE_PAIRS)                 /* [rad/s]->[rpm] : scale(2^10 / 2^2 = 2^8) */

#define MTR_SPEED_LIMIT_RPM     (4290)                                  /* over speed limit [rpm](mechanical angle) : scale(2^0) */
#define MTR_SPEED_LIMIT         ((int16_t)((MTR_SPEED_LIMIT_RPM / 60) * MTR_POLE_PAIRS * MTR_TWOPI * 4))
                                                                        /* over speed limit [rad/s](electrical angle) : scale(2^2) */
#define MTR_SPEED_LPF_K         ((int16_t)(CP_SPEED_LPF_K * 16384))     /* speed LPF gain : scale(2^14) */

#define MTR_SPEED_CALC_BASE     (383)                                   /* conversion factor (rad/s <-> timer count) */
                                                                        /* 1000000 * 2PI / 2^16 * 2^2 : scale(2^-14) */

#define MTR_TIMER_CNT_BUF_NUM   (6)                                     /* buffer number of counts for 60 degrees */

#define MTR_TIMEOUT_CNT         (400)                                   /* undetected time = MTR_TIMEOUT_CNT * 50[us] */
#define MTR_START_CNT           (2000)                                  /* boot mode time = MTR_START_CNT * 500[us] */

/* value of hall sensor (CW) */
#define MTR_PATTERN_CW_V_U      (5)                                     /* V phase to U phase */
#define MTR_PATTERN_CW_W_U      (4)                                     /* W phase to U phase */
#define MTR_PATTERN_CW_W_V      (6)                                     /* W phase to V phase */
#define MTR_PATTERN_CW_U_V      (2)                                     /* U phase to V phase */
#define MTR_PATTERN_CW_U_W      (3)                                     /* U phase to W phase */
#define MTR_PATTERN_CW_V_W      (1)                                     /* V phase to W phase */

/* value of hall sensor (CCW) */
#define MTR_PATTERN_CCW_V_U     (2)                                     /* V phase to U phase */
#define MTR_PATTERN_CCW_V_W     (6)                                     /* V phase to W phase */
#define MTR_PATTERN_CCW_U_W     (4)                                     /* U phase to W phase */
#define MTR_PATTERN_CCW_U_V     (5)                                     /* U phase to V phase */
#define MTR_PATTERN_CCW_W_V     (1)                                     /* W phase to V phase */
#define MTR_PATTERN_CCW_W_U     (3)                                     /* W phase to U phase */

/* voltage pattern */
#define MTR_PATTERN_ERROR       (0)                                     /* hall pattern error */
#define MTR_U_PWM_VN_ON         (1)                                     /* U(PWM) to Vn(on) */
#define MTR_U_PWM_WN_ON         (2)                                     /* U(PWM) to Wn(on) */
#define MTR_V_PWM_UN_ON         (3)                                     /* V(PWM) to Un(on) */
#define MTR_V_PWM_WN_ON         (4)                                     /* V(PWM) to Wn(on) */
#define MTR_W_PWM_UN_ON         (5)                                     /* W(PWM) to Un(on) */
#define MTR_W_PWM_VN_ON         (6)                                     /* W(PWM) to Vn(on) */
#define MTR_UP_ON_V_PWM         (7)                                     /* Up(on) to V(PWM) */
#define MTR_UP_ON_W_PWM         (8)                                     /* Up(on) to W(PWM) */
#define MTR_VP_ON_U_PWM         (9)                                     /* Vp(on) to U(PWM) */
#define MTR_VP_ON_W_PWM         (10)                                    /* Vp(on) to W(PWM) */
#define MTR_WP_ON_U_PWM         (11)                                    /* Wp(on) to U(PWM) */
#define MTR_WP_ON_V_PWM         (12)                                    /* Wp(on) to V(PWM) */

/* rotational direction */
#define MTR_CW                  (0)
#define MTR_CCW                 (1)

#define MTR_FLG_CLR             (0)                                     /* for flag clear */
#define MTR_FLG_SET             (1)                                     /* for flag set */
#define MTR_STOP_WAIT_CNT       (200)
/* Timer RD interrupt period(50[us]) * MTR_STOP_WAIT_CNT = motor stop waiting time */

#define MTR_ICS_DECIMATION      (4)                                     /* ICS decimation count */

/******************************************************************************
* Macro definitions for sequence control
******************************************************************************/
/* run mode */
#define MTR_BOOT_MODE           (0x00)
#define MTR_OPENLOOP_MODE       (0x01)
#define MTR_START_MODE          (0x02)
#define MTR_HALL_120_MODE       (0x03)
#define MTR_BEMF_120_MODE       (0x04)
#define MTR_ENCD_FOC_MODE       (0x05)
#define MTR_LESS_FOC_MODE       (0x06)

/* error status */
#define MTR_ALARM_ERROR         (0x01)
#define MTR_OVER_VOLTAGE_ERROR  (0x02)
#define MTR_OVER_SPEED_ERROR    (0x04)
#define MTR_TIMEOUT_ERROR       (0x08)
#define MTR_HALL_ERROR          (0x10)
#define MTR_BEMF_ERROR          (0x20)
#define MTR_UNDER_VOLTAGE_ERROR (0x40)
#define MTR_UNKNOWN_ERROR       (0x80)
#define MTR_HW_ERROR            (MTR_ALARM_ERROR)
#define MTR_FW_ERROR            (MTR_OVER_VOLTAGE_ERROR | MTR_OVER_SPEED_ERROR    | MTR_TIMEOUT_ERROR |\
                                 MTR_HALL_ERROR         | MTR_UNDER_VOLTAGE_ERROR | MTR_UNKNOWN_ERROR)

/* system mode */
#define MTR_MODE_STOP           (0x00)
#define MTR_MODE_RUN            (0x01)
#define MTR_MODE_ERROR          (0x02)
#define MTR_SIZE_STATE          (3)                                     /* number of state */

/* event */
#define MTR_EVENT_STOP          (0x00)
#define MTR_EVENT_RUN           (0x01)
#define MTR_EVENT_ERROR         (0x02)
#define MTR_EVENT_RESET         (0x03)
#define MTR_SIZE_EVENT          (4)                                     /* number of event */

typedef struct
{
    int16_t    s2_ref_speed;
    int16_t    s2_direction;
    int16_t    s2_speed_lpf_k;
} MTR_ICS_INPUT;

/******************************************************************************
* Global function definitions
******************************************************************************/
void R_MTR_IcsInput(MTR_ICS_INPUT *);                                   /* set value to buffer */
void R_MTR_SetSpeed(int16_t);                                           /* set speed reference */
void R_MTR_SetDir(uint8_t);                                             /* set rotational direction */
int16_t R_MTR_GetSpeed(void);                                           /* get speed value */
uint8_t R_MTR_GetDir(void);                                             /* get rotation direction */
uint8_t R_MTR_GetStatus(void);                                          /* get status of motor control system */

void mtr_pattern_set(void);                                             /* set voltage pattern */
void mtr_set_variables(void);                                           /* set variables (conversion of unit) */
void mtr_error_check(void);                                             /* check error status */

/* for sequence control */
void R_MTR_InitSequence(void);                                          /* initialize variables for sequence control */
void R_MTR_ExecEvent(uint8_t);                                          /* change motor state and select action */

#endif /* MTR_SPM_HALL_120_CPM_H */
